@@ -1,3 +1,18 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.asm;
 
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
@@ -12,6 +27,7 @@ import net.bytebuddy.matcher.ElementMatcher;
 import net.bytebuddy.pool.TypePool;
 import net.bytebuddy.utility.CompoundList;
 import net.bytebuddy.utility.OpenedClassReader;
+import net.bytebuddy.utility.nullability.MaybeNull;
 import org.objectweb.asm.ClassVisitor;
 import org.objectweb.asm.FieldVisitor;
 import org.objectweb.asm.MethodVisitor;
@@ -297,7 +313,9 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
                 ModifierContributor.Resolver.of(modifierContributors)), methodAdjustments));
     }
 
-    @Override
+    /**
+     * {@inheritDoc}
+     */
     public ModifierAdjustingClassVisitor wrap(TypeDescription instrumentedType,
                                               ClassVisitor classVisitor,
                                               Implementation.Context implementationContext,
@@ -352,8 +370,10 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
             this.resolver = resolver;
         }
 
-        @Override
-        public boolean matches(T target) {
+        /**
+         * {@inheritDoc}
+         */
+        public boolean matches(@MaybeNull T target) {
             return matcher.matches(target);
         }
 
@@ -431,7 +451,7 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
         }
 
         @Override
-        public void visit(int version, int modifiers, String internalName, String signature, String superClassName, String[] interfaceName) {
+        public void visit(int version, int modifiers, String internalName, @MaybeNull String signature, @MaybeNull String superClassName, @MaybeNull String[] interfaceName) {
             for (Adjustment<TypeDescription> adjustment : typeAdjustments) {
                 if (adjustment.matches(instrumentedType)) {
                     modifiers = adjustment.resolve(modifiers);
@@ -442,7 +462,7 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
         }
 
         @Override
-        public void visitInnerClass(String internalName, String outerName, String innerName, int modifiers) {
+        public void visitInnerClass(String internalName, @MaybeNull String outerName, @MaybeNull String innerName, int modifiers) {
             if (instrumentedType.getInternalName().equals(internalName)) {
                 for (Adjustment<TypeDescription> adjustment : typeAdjustments) {
                     if (adjustment.matches(instrumentedType)) {
@@ -455,7 +475,8 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
         }
 
         @Override
-        public FieldVisitor visitField(int modifiers, String internalName, String descriptor, String signature, Object value) {
+        @MaybeNull
+        public FieldVisitor visitField(int modifiers, String internalName, String descriptor, @MaybeNull String signature, @MaybeNull Object value) {
             FieldDescription.InDefinedShape fieldDescription = fields.get(internalName + descriptor);
             if (fieldDescription != null) {
                 for (Adjustment<FieldDescription.InDefinedShape> adjustment : fieldAdjustments) {
@@ -469,7 +490,8 @@ public class ModifierAdjustment extends AsmVisitorWrapper.AbstractBase {
         }
 
         @Override
-        public MethodVisitor visitMethod(int modifiers, String internalName, String descriptor, String signature, String[] exception) {
+        @MaybeNull
+        public MethodVisitor visitMethod(int modifiers, String internalName, String descriptor, @MaybeNull String signature, @MaybeNull String[] exception) {
             MethodDescription methodDescription = methods.get(internalName + descriptor);
             if (methodDescription != null) {
                 for (Adjustment<MethodDescription> adjustment : methodAdjustments) {
